;-----------------------------
;   ZAZNAMENAVANI IR-KODU
;-----------------------------

; zaznamenava IR-KOD do staticke RAM pameti
; - podprogram pracuje tak, ze opakovane prijima novou hodnotu a
;   nakonec sam prohlasi, ze je zaznam hotov
IR_REC_	MACRO	KOD_, POZICE_, BLOK1, BLOK2, MINPOCET, MAXPOCET
; KOD_ - vstupni 16bitova hodnota (promenna)
; POZICE_ - relativni ukazatel do zaznamu (promenna)
; BLOK1 - (konstanta) adresa zacatku bloku v pameti RAM, do ktere
;         se budou ukladat prvni byty vstupnich hodnot
; BLOK2 - (konstanta) adresa zacatku bloku v pameti RAM, do ktere
;         se budou ukladat druhe byty vstupnich hodnot
; MINPOCET - (konstanta) minimalni pripustna delka kodu
; MAXPOCET - (konstanta) maximalni pripustna delka kodu
	LOCAL	NULA, NENULA, HOTOVO, KONEC

	MOVF	KOD_,F	; testujeme, zda je KOD_ nulovy
	BTFSS	STATUS,Z
	GOTO	NENULA
	MOVF	KOD_+1,F
	BTFSC	STATUS,Z	
	GOTO	NULA
NENULA	MOVF	POZICE_,W	; nastavime pozici zapisu prvniho bytu
	ADDLW	BLOK1
	bankisel	BLOK1
	MOVWF	FSR
	MOVF	KOD_,W	; zapiseme prvni byte
	MOVWF	INDF
	MOVF	POZICE_,W	; nastavime pozici zapisu druheho bytu
	ADDLW	BLOK2
	bankisel	BLOK2
	MOVWF	FSR
	MOVF	KOD_+1,W	; zapiseme druhy byte
	MOVWF	INDF
	INCF	POZICE_,F	; posunuti (zvyseni)  ukazatele
	MOVF	POZICE_,W
	SUBLW	MAXPOCET - D'1'	; testovani maximalni delky kodu
	BTFSS	STATUS,C
	GOTO	HOTOVO	; pokud je kod maximalne dlouhy, ukoncime zaznam
	BCF	STATUS,C	; v opacnem pripade indikujeme, ze zaznam neni hotov
	GOTO	KONEC	; ...a provedeme navrat
NULA	MOVLW	MINPOCET	; (vstupni hodnota je nulova)
	SUBWF	POZICE_,W	; testujeme, zda je zapsany kod dostatecne dlouhy
	BTFSC	STATUS,C
	GOTO	HOTOVO	; pokud ano, tak ukoncime zaznam
	CLRF	POZICE_	; v opacnem pripade smazeme zaznam, a zaciname od zacatku
	BCF	STATUS,C	; indikujeme, ze zaznam dosud neskoncil
	GOTO	KONEC
HOTOVO	MOVF	POZICE_,W	; (ukonceni zaznamu)
	ADDLW	BLOK1	; nastavujeme ukazatel do pameti na konec zaznamu...
	bankisel	BLOK1
	MOVWF	FSR
	CLRF	INDF	; ukladame nulu (ukoncovaci znak)
	MOVF	POZICE_,W	; totez provedeme pro druhy byte...
	ADDLW	BLOK2
	bankisel	BLOK2
	MOVWF	FSR
	CLRF	INDF
	BSF	STATUS,C	; indikujeme, ze zaznam je hotov!!!
KONEC
	ENDM

; slouzi ke cteni zaznamu z RAM pameti
READ_REC_	MACRO	KOD_, POZICE_, BLOK1, BLOK2
; KOD_ - vytupni 16bitova hodnota (promenna)
; POZICE_ - relativni ukazatel do zaznamu (promenna)
; BLOK1 - (konstanta) adresa zacatku bloku v pameti RAM, ve ktere
;         se nachazi prvni byty hodnot
; BLOK2 - (konstanta) adresa zacatku bloku v pameti RAM, ve ktere
;         se nachazi druhe byty hodnot
	
	MOVF	POZICE_,W	; precteme aktualni hodnotu ze zaznamu:
	ADDLW	BLOK1
	bankisel	BLOK1
	MOVWF	FSR
	MOVF	INDF,W	; - prvni byte
	MOVWF	KOD_
	MOVF	POZICE_,W
	ADDLW	BLOK2
	bankisel	BLOK2
	MOVWF	FSR
	MOVF	INDF,W	; - druhy byte
	MOVWF	KOD_+1
	INCF	POZICE_,F	; zvysime ukazatel
	
	ENDM

; slouzi ke cteni zaznamu z FLASH pameti
FLRD_REC_	MACRO	KOD_, POZICE_, K_ADR_
; KOD_ - vytupni 16bitova hodnota (promenna)
; POZICE_ - relativni ukazatel do zaznamu (promenna)
; K_ADR_ - (pozor-promenna!!) 16bitova adresa zaznamu ve FLASH pameti

	MOVF	POZICE_,W	; precteme aktualni hodnotu ze zaznamu:
	ADDWF	K_ADR_,W
	banksel	EEADR
	MOVWF	EEADR
	banksel	K_ADR_+1
	MOVF	K_ADR_+1,W
	BTFSC	STATUS,C
	INCF	K_ADR_+1,W
	banksel	EEADRH
	MOVWF	EEADRH
	FREAD_		; proces cteni z FLASH
	banksel	EEDATA
	MOVF	EEDATA,W
	banksel	KOD
	MOVWF	KOD
	banksel	EEDATH
	MOVF	EEDATH,W
	banksel	KOD+1
	MOVWF	KOD+1	
	INCF	POZICE_,F	; zvyseni relativniho ukazatele

	ENDM

; zkopiruje zaznam z RAM pameti do FLASH pameti
SAVE_REC_	MACRO	KOD_, POZICE_, BLOK1, BLOK2, K_ADR_
; KOD_ - vytupni 16bitova hodnota (promenna)
; POZICE_ - relativni ukazatel do zaznamu (promenna)
; BLOK1 - (konstanta) adresa zacatku bloku v pameti RAM, ve ktere
;         se nachazi prvni byty hodnot
; BLOK2 - (konstanta) adresa zacatku bloku v pameti RAM, ve ktere
;         se nachazi druhe byty hodnot
; K_ADR_ - (pozor-promenna!!) 16bitova adresa zaznamu ve FLASH pameti
	LOCAL	LOOP,NENULA,KONEC
	CLRF	POZICE_	; nastaveni relativniho ukazatele na zacatek
LOOP
	READ_REC_	KOD_, POZICE_, BLOK1, BLOK2	; precteni aktualni hodnoty
	MOVF	KOD_,F	; testovani nuly
	BTFSS	STATUS,Z
	GOTO	NENULA
	MOVF	KOD_+1,F
	BTFSC	STATUS,Z
	GOTO	KONEC	; pokud byla zjistena nula, cyklus konci
NENULA	DECF	POZICE_,W	; nastaveni FLASH adresy (v promenne POZICE_ je jiz inkrementovana hodnota)
	ADDWF	K_ADR_,W
	banksel	EEADR
	MOVWF	EEADR
	banksel	K_ADR_
	MOVF	K_ADR_+1,W
	BTFSC	STATUS,C
	INCF	K_ADR_+1,W
	banksel	EEADRH
	MOVWF	EEADRH
	banksel	KOD	; priprava hodnoty pro zapis
	MOVF	KOD,W
	banksel	EEDATA
	MOVWF	EEDATA
	banksel	KOD+1
	MOVF	KOD+1,W
	banksel	EEDATH
	MOVWF	EEDATH
	FWRITE_	1,1	; provedeni zapisu (docasna deaktivace preruseni + ochrana zavadece)
	GOTO	LOOP	; cyklus se opakuje...
KONEC	DECF	POZICE_,W	; ukonceni kopirovani:
	ADDWF	K_ADR_,W	; na konec zaznamu ulozime nulu
	banksel	EEADR
	MOVWF	EEADR
	banksel	K_ADR_
	MOVF	K_ADR_+1,W
	BTFSC	STATUS,C
	INCF	K_ADR_+1,W
	banksel	EEADRH
	MOVWF	EEADRH
	banksel	EEDATA
	CLRF	EEDATA
	CLRF	EEDATH
	FWRITE_	1,1

	ENDM
	
