;--------------------------------------------;
;  TENTO PROGRAM OVLADA POCITAC NA ZAKLADE   ;
;      POVELU Z DALKOVEHO IR-OVLADACE.       ;
;                                            ;
;    Musite ho naucit dany ovladac a dane    ;
;           klavesove zkratky.               ;
;                                            ;
;      Pripojte Chipon k PC a klavesnici     ;
;             pomoci rele-modulu !           ;
;--------------------------------------------;

	LIST	P = 16F877, R = DEC
	#include	p16f877.inc

	__CONFIG	B'11111100111010'

	#include	displej.inc
	#include	unibuff.inc
	#include	intervl2.inc
	#include	ir_rec2.inc
	#include	flash.inc
	#include	tab_util.inc
	#include	dskripty.inc
	#include	div16u.inc
	#include	vyber.inc
	#include	vyber2.inc
	#include	key_rec.inc
	#include	keymodul.inc

;---------------------------------------------
;||||||||||| KONSTANTY PROGRAMU ||||||||||||||
;---------------------------------------------

MINPULZ	EQU	D'150'	; maximalni delka jednoznacne poruchy
ADIT_CH	EQU	D'100'

BLOK1	EQU	H'110'	; vyber zacatku prvni RAM oblasti
BLOK2	EQU	H'190'	; vyber zacatku druhe RAM oblasti
CK_ADR	EQU	D'2048'	; adresa prvniho IR-zaznamu
KEY_ADR	EQU	D'4096'	; adresa prvniho klavesoveho zaznamu
BUFF_SIZE	EQU	D'64'	; velikost prijimaciho IR-bufferu

MINKOD	EQU	D'6'	; minimalni delka IR_kodu
MAXKOD	EQU	D'64'	; maximalni delka IR_kodu
MAXKKOD	EQU	D'64'	; maximalni delka klavesove posl.

POCETKODU	EQU	D'10'	; pocet dvojic (IR-kod, klav.posl.)

OFFRAM	EQU	H'20'	; zacatek volne pameti "RAM"
TEMPRAM	EQU	H'70'	; posledni blok pameti (16 bytu)
			; - je zrcadlen do vsech pametovych bank

#DEFINE	DB4	PORTE,2	; piny pro ovladani displeje
#DEFINE	DB5	PORTE,1
#DEFINE	DB6	PORTE,0
#DEFINE	DB7	PORTA,4
#DEFINE	E	PORTB,1
#DEFINE	RW	PORTB,2
#DEFINE	RS	PORTB,3

; PRO PC-KLAVESNICI
KEYPORT	EQU	PORTC	; vybrany klavesovy port
KEYMASK	EQU	B'11001111'; inicializacni "AND" maska portu
#DEFINE	CLOCK	KEYPORT,4	; hodinovy vstup/vystup klavesnice
#DEFINE	KEYDATA	KEYPORT,5	; datovy vstup/vystup klavesnice
#DEFINE	PREPINAC	KEYPORT,3	; spinac vodicu klavesnice
TLACPORT	EQU	PORTD	; port 16tlac. klavesnice
#DEFINE	INC_KEY	TLACPORT,0; inkrementacni tlacitko
#DEFINE	OK_KEY	TLACPORT,1; potvrzovaci tlacitko
#DEFINE	IR_KEY	TLACPORT,2; volba modu IR_zaznamu
#DEFINE	KLAV_KEY	TLACPORT,3; volba modu klavesnicoveho zaznamu
#DEFINE	KBREAK	TLACPORT,2; tlacitko preruseni klav. posloupnosti
;--------------------------

	CBLOCK	OFFRAM
	TML:1	; pro obsluhu displeje
	TMH:1
	DDATA:1

	BUFF_IN:1	; pro obsluhu bufferu
	BUFF_OUT:1
	BUFF_STAT:1
	BUFF_TEMP:1

	POZICE:1	; zaznam IR-kodu do RAM
	KOD:2
	I1:2
	I2:2

	K_ADR:2	; promenna FLASH adresa

	AA:2	; pro skripty
	BB:2
	CC:2
	VD16U:1
	NUMFORM_:1

	KINDEX:1	; indexace zaznamu
	KINDEXY:POCETKODU

	KEYCOUNT:1 ; prace s PC klavesnici
	KEYKOD:2
	TIME:2

	KKOD:2	; klavesnicove zaznamy
	KPOZICE:1
	MAXPOZICE:1

	REC_ADR:2	; volba klavesove zkratky
	INDEX:1

	ENDC

	CBLOCK	H'A0'	; usazeni bufferu do zbyle volne pameti
	BUFF_ADR:BUFF_SIZE
	ENDC

	CBLOCK	TEMPRAM	; zalozni registry, ktere
	W_TEMP:1		; jsou nezavisle na zvolene
	ST_TEMP:1		; pametove bance !!!
	FSR_TEMP:1
	ENDC

;------------------------------------------------
; RESET / START	

	ORG	0
	NOP	; kvuli ZAVADECI
	NOP	; kvuli ZAVADECI
	NOP	; kvuli ZAVADECI
	GOTO	START

;------------------------------------------------
; OBSLUHA PRERUSENI

	ORG	4
	MOVWF	W_TEMP		; zaloha registru W,STATUS a FSR
	MOVF	STATUS,W
	MOVWF	ST_TEMP
	MOVF	FSR,W
	MOVWF	FSR_TEMP
	banksel	0
	BTFSS	PIR1,TMR1IF	; pokud doslo k preteceni casovace
	GOTO	RB_INT
	MOVLW	D'0'	; ulozime do bufferu nulovou hodnotu (16 bitu)
	CALL	IN_BUFF
	MOVLW	D'0'
	CALL	IN_BUFF
	banksel	PIE1	; nasledne zakazeme dalsi preruseni od casovace
	BCF	PIE1,TMR1IE
	banksel	0
RB_INT	BTFSS	INTCON,RBIF	; testujeme preruseni od optoprijimace
	GOTO	INT_END
	MOVF	PORTB,F		; "refresujeme" porovnavaci registr
	BCF	INTCON,RBIF	; vynulujeme priznak
	BCF	T1CON,TMR1ON	; zastavime TMR1
	banksel	PIE1
	BSF	PIE1,TMR1IE	; povolime prerusemi od casovace
	banksel	PIR1
	BTFSS	PIR1,TMR1IF	; pokud TMR1 pretekl,...
	GOTO	VYSLI_H
	CLRF	TMR1H		; ..., potom jej vynulujeme
	CLRF	TMR1L
	BCF	PIR1,TMR1IF
VYSLI_H	MOVF	TMR1H,W		; vyssi byte TMR1 vlozime do bufferu
	CALL	IN_BUFF
	MOVF	TMR1H,F		; pokud je vyssi byte nulovy,...
	BTFSS	STATUS,Z
	GOTO	SKOK
	MOVF	TMR1L,W		; ...potom testujeme zda je nulova
	BTFSC	STATUS,Z
	GOTO	SKOK
	SUBLW	MINPULZ		; pokud neni nulova,...
	BTFSS	STATUS,C		; ...testujeme, zda neni prilis mala
	GOTO	SKOK
	CLRF	TMR1L		; pokud je nenulova, ale mala,...
	BSF	PIR1,TMR1IF	; ...potom predstirame chybu v budoucim mereni
SKOK	MOVF	TMR1L,W		; zde vkladame nizsi byte TMR1 do bufferu
	CALL	IN_BUFF
	CLRF	TMR1H		; nulujeme TMR1
	CLRF	TMR1L
	BSF	T1CON,TMR1ON	; TMR1 je spusten

INT_END	MOVF	FSR_TEMP,W	; obnova registru ze zalohy
	MOVWF	FSR
	MOVF	ST_TEMP,W
	MOVWF	STATUS
	SWAPF	W_TEMP,F
	SWAPF	W_TEMP,W
	RETFIE

;-------------------------------------
; INICIALIZACE
	
START	
	banksel	OPTION_REG
	BCF	OPTION_REG,T0CS	; zdroj pulzu pro TMR0
	BCF	OPTION_REG,PSA	; nasataveni predelicky TMR0 (1:16)
	BCF	OPTION_REG,2
	BSF	OPTION_REG,1
	BSF	OPTION_REG,0 
	banksel	PORTC
	BCF	PREPINAC	; dopredne nastaveni rele-spinace
	banksel	TRISC
	BCF	PREPINAC	; aktivace rele-spinace
	BSF	CLOCK	; klavesnicove hodiny jako vstup
	BSF	KEYDATA	; klavesnicova data jako vstup

	banksel	TRISB
	BCF	TRISB,4	; musi byt nastaveno jako vystup,...
	BCF	TRISB,5	; ...aby nebyl generovan priznak RBIF
	BCF	TRISB,6
	BSF	TRISB,7	; sem je pripojen optoprijimac

	banksel	PORTD	; priprava 16tlac. klavesnice na pasivni 4 tlac. rezim
	MOVLW	B'00001111'
	MOVWF	PORTD
	banksel	TRISD	; ridici vodice jsou v rezimu vystupu,...
	MOVWF	TRISD	; ...prijimaci vodice jsou v rezimu vstupu

	MOVLW	B'00100100'	; nastav vysilac pro...
	banksel	TXSTA		; ...asynchronni 8bitovy prenos
	MOVWF	TXSTA
	MOVLW	B'10010000'	; nastav prijimac pro...
	banksel	RCSTA		; ...asynchronni 8bitovy prijem
	MOVWF	RCSTA
	MOVLW	D'10'		; nastav komunikacni rychlost,...
	banksel	SPBRG		; ...v nasem pripade 19200 baudu
	MOVWF	SPBRG
	banksel	ADCON1
	MOVLW	B'00000110'	; nastav PORTA a PORTE jako "logicky"
	MOVWF	ADCON1
	banksel	0
	; nastaveni TMR1 (predelicka 1:4);
	banksel	T1CON	
	MOVLW	B'00101101'
	MOVWF	T1CON
	; inicializace displeje
	CALL	INITDISP	

;-----------------------------------------------------------
; HLAVNI PROGRAM

	BTFSS	IR_KEY
	GOTO	IR_ZAZNAM
	BTFSS	KLAV_KEY
	GOTO	KL_ZAZNAM
GOBACK	; inicializace bufferu
	BUFFINI 	BUFF_IN, BUFF_OUT,BUFF_STAT
	BSF	INTCON,RBIE	; povoleno preruseni od optoprijimace
	BSF	INTCON,PEIE	; povoleno preruseni od nektere periferie (v tomto pripade TMR1)
	BSF	INTCON,GIE	; aktivace systemu preruseni
	MOVLW	D'2'	; napis: "otestujte prijem"
	CALL	SKRIPT
	INDEXNUL_	KINDEX, POCETKODU, KINDEXY ; vynulovani vsech indexu (tedy pro vsechny zaznamy)
LOOP2	MOVF	BUFF_STAT,F
	BTFSC	STATUS,Z	; pokud je v bufferu hodnota...
	GOTO	LOOP2
	CALL	OUT_BUFF	; precteme ji...
	MOVWF	KOD+1
	CALL	OUT_BUFF
	MOVWF	KOD
	KRATIT_14	KOD	; !!!zkraceni KODu na 14 bitu!!!
	INTERVAL_	KOD, I1, I2, ADIT_CH	; spocteme tolerancni interval
LOOP1_
	PRVNI_K	CK_ADR, K_ADR, KINDEX, KINDEXY	; nastaveni na prvni zaznam
LOOP2_	CALL	FLRD_REC	; precteme aktualni hodnotu ze zaznamu
	MOVF	KOD,F	; testujeme zda je hodnota nulova
	BTFSS	STATUS,Z
	GOTO	POROVNEJ
	MOVF	KOD+1,F
	BTFSC	STATUS,Z
	GOTO	POZNAL	; - pokud je nulova, znamena to, ze jsme na konci zaznamu a doslo k celkove shode
POROVNEJ			; - v opacnem pripade porovnavame...
	UVNITR_	KOD, I1, I2
	BTFSS	STATUS,C	; lezi KOD v intervalu?
	CLRF	INDF	; - pokud ne, resetujeme ukazatel do zaznamu
	DALSI_K	CK_ADR, K_ADR, BUFF_SIZE, KINDEX, POCETKODU, KINDEXY ; prejdeme na dalsi zaznam
	BTFSC	STATUS,Z	; pokud byl predchozi zaznam posledni...
	GOTO	LOOP2	; ...potom musime prijmout z bufferu dalsi hodnotu
	GOTO	LOOP2_	; ...v opacnem pripade musime porovnat hodnotu z dalsiho zaznamu

POZNAL	MOVLW	D'133'	; kod byl rozpoznan...
	BTFSS	PIR1,TXIF	; na port vysleme cislo indexu, jehoz zaznam byl rozpoznan
	GOTO	$ - 1
	MOVWF	TXREG
	MOVLW	D'0'
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	MOVF	KINDEX,W
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	CLRF	INDF	; nulujeme aktualni ukazatel do zaznamu
	MOVLW	D'3'
	CALL	SKRIPT
	MOVF	KINDEX,W
	MOVWF	INDEX
	CALL	RUN_KEY	; volani podprogramu simulace klavesnice
	DALSI_K	CK_ADR, K_ADR, BUFF_SIZE, KINDEX, POCETKODU, KINDEXY	; prejdeme na dalsi zaznam
	BTFSC	STATUS,Z	; pokud byl predchozi zaznam posledni...
	GOTO	LOOP2	; ...potom musime prijmout z bufferu dalsi hodnotu
	GOTO	LOOP2_	; ...v opacnem pripade musime porovnat hodnotu z dalsiho zaznamu

; ukladani do bufferu
IN_BUFF	INBUFF_	BUFF_ADR, BUFF_SIZE, BUFF_IN, BUFF_OUT, BUFF_STAT, BUFF_TEMP, 0
	RETURN
	
; cteni z bufferu
OUT_BUFF	OUTBUFF_	BUFF_ADR, BUFF_SIZE, BUFF_IN, BUFF_OUT, BUFF_STAT, BUFF_TEMP, 1
	RETURN

; ukladani IR-kodu do pameti
IR_REC	IR_REC_	KOD, POZICE, BLOK1, BLOK2, MINKOD, MAXKOD
	RETURN

; cteni IR-kodu z flash pameti
FLRD_REC	FLRD_REC_	KOD, INDF, K_ADR
	RETURN

; 16bitove deleni
DIV16U	DIV16U_M	AA,BB,CC,VD16U 

; zobrazovaci skripty
SKRIPT	SKRIPT_	TABULKY, PUTZNAK, DPRIKAZ, NUMFORM_, DIV16U, AA, BB, CC
	RETURN

; ukladani zaznamu klav. posloupnosti
KEY_REC	KEY_REC_	KKOD, KPOZICE, BLOK1, BLOK2, MAXKKOD
	RETURN

; cteni zaznamu klav. posloupnosti
KREAD_REC	KRD_REC_	KKOD, KPOZICE, BLOK1, BLOK2, MAXPOZICE
	RETURN

; cteni z flash (vcetne autoinkrementace)
FREAD	FREAD_
	banksel	EEADR
	INCF	EEADR,F
	BTFSC	STATUS,Z
	INCF	EEADRH,F
	banksel	0 
	RETURN

; zapis do flash (vcetne autoinkrementace)
FWRITE	FWRITE_	0,1
	banksel	EEADR
	INCF	EEADR,F
	BTFSC	STATUS,Z
	INCF	EEADRH,F
	banksel	0 
	RETURN

; obsluha displeje
	DISPLEJ	TMH,TML,DDATA,DB4,DB5,DB6,DB7,E,RW,RS

; prijem kodu klavesy a casu	
PRIJMOUT	PRIJMOUT_

; vyslani kodu klavesy
VYSLIKEY	VYSLIKEY_

; generovani 16bitove prodlevy
CEKEJ	CEKEJ_

;--------------------
; podprogram zaznamenavajici IR-posloupnost
;--------------------
IR_ZAZNAM	; inicializace bufferu
	BUFFINI 	BUFF_IN, BUFF_OUT,BUFF_STAT
	; napis: "vygenerujte novy IR-kod"
	MOVLW	D'0'
	CALL	SKRIPT
	BSF	INTCON,RBIE	; povoleno preruseni od optoprijimace
	BSF	INTCON,PEIE	; povoleno preruseni od nektere periferie (v tomto pripade TMR1)
	BSF	INTCON,GIE	; aktivace systemu preruseni
	CLRF	POZICE	; ukazatel do zaznamu je nastaven na prvni polozku
IR_LOOP	MOVF	BUFF_STAT,F
	BTFSC	STATUS,Z	; pokud je v bufferu hodnota,... 
	GOTO	IR_LOOP
	CALL	OUT_BUFF	; ...ulozime ji do promenne KOD
	MOVWF	KOD+1
	CALL	OUT_BUFF
	MOVWF	KOD
	KRATIT_14	KOD	; !!!zkraceni KODu na 14 bitu!!!
	CALL	IR_REC	; volame zaznamenavaci podprogram
	BTFSS	STATUS,C	; je zaznam hotov?
	GOTO	IR_LOOP	; - pokud ne, opakujeme cyklus
	; uzivatel vybere index zaznamu
	VYBER_	CK_ADR, K_ADR, BUFF_SIZE, KINDEX, D'10', SKRIPT, D'1', DDELAY
	; novy zaznam se ulozi bo zvolene oblasti (viz. index)
	SAVE_REC_	KOD, POZICE, BLOK1, BLOK2, K_ADR
	GOTO	GOBACK

;----------------------------
; blok programu, ktery se stara o zaznam klavesove posloupnosti do flash
;----------------------------
; prijimani posloupnosti (viz. minuly program) 
KL_ZAZNAM	MOVLW	D'4'
	CALL	SKRIPT
	CLRF	KPOZICE
KL_LOOP	CALL	PRIJMOUT
	BTFSC	STATUS,Z
	GOTO	KONECREC
	MOVLW	D'155'
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	MOVF	TIME+1,W
	MOVWF	KKOD+1
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	MOVF	TIME,W
	MOVWF	KKOD
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	CALL	KEY_REC
	BTFSS	STATUS,C
	GOTO	PRYC
	MOVLW	D'133'
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	MOVF	KEYKOD+1,W
	MOVWF	KKOD+1
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	MOVF	KEYKOD,W
	MOVWF	KKOD
	BTFSS	PIR1,TXIF
	GOTO	$ - 1
	MOVWF	TXREG
	CALL	KEY_REC
	BTFSC	STATUS,C
	GOTO	KL_LOOP	; navrat k testovani bufferu
	GOTO	PRYC
KONECREC	INCF	KPOZICE,F
; ZAZNAM DO FLASH (viz. minuly program)
PRYC	MOVF	KPOZICE,W
	MOVWF	MAXPOZICE
	CLRF	KPOZICE
	VYBER_	KEY_ADR,REC_ADR,D'64',INDEX,D'10',SKRIPT,D'5',DDELAY
	MOVF	REC_ADR+1,W
	banksel	EEADRH
	MOVWF	EEADRH
	banksel	REC_ADR
	MOVF	REC_ADR,W
	banksel	EEADR
	MOVWF	EEADR
	banksel	0

	CALL	KREAD_REC
	BTFSC	STATUS,Z
	GOTO	KL_SKOK0
KL_LOOP2	CALL	KREAD_REC
	BTFSC	STATUS,Z
	GOTO	KL_SKOK0
	MOVF	KKOD,W
	banksel	EEDATA
	MOVWF	EEDATA
	banksel	KKOD+1
	MOVF	KKOD+1,W
	banksel	EEDATH
	MOVWF	EEDATH
	banksel	0
	CALL	FWRITE

	CALL	KREAD_REC
	BTFSC	STATUS,Z
	GOTO	KL_SKOK0

	MOVF	KKOD,W
	banksel	EEDATA
	MOVWF	EEDATA
	banksel	KKOD+1
	MOVF	KKOD+1,W
	banksel	EEDATH
	MOVWF	EEDATH
	banksel	0
	CALL	FWRITE

	MOVF	KKOD,F
	BTFSS	STATUS,Z
	GOTO	KL_LOOP2
	MOVF	KKOD+1,F
	BTFSC	STATUS,Z
KL_SKOK0	
	banksel	EEDATA
	CLRF	EEDATA
	CLRF	EEDATH
	CALL	FWRITE
	banksel	0
	MOVLW	D'1'
	CALL	SKRIPT
	GOTO	GOBACK

;-------------------
; PROVADI PREHRANI POSLOUPNOSTI S INDEXEM "INDEX" (SIMULACE KLAVESNICE)
;-------------------
RUN_KEY	VYBER2_	KEY_ADR,REC_ADR,D'64',INDEX
	BSF	PREPINAC	; odpojeni klavesnice
	CLRF	TMR0
	BCF	INTCON,T0IF
	BTFSS	INTCON,T0IF
	GOTO	$ - 1
	BCF	INTCON,GIE	; zakaz preruseni!
	BTFSC	INTCON,GIE
	GOTO	$ - 2
	MOVF	REC_ADR+1,W	; nastaveni flash adresy
	banksel	EEADRH
	MOVWF	EEADRH
	banksel	REC_ADR
	MOVF	REC_ADR,W
	banksel	EEADR
	MOVWF	EEADR
	banksel	0
	; probiha prehravani (viz. minuly program), ale bez vysilani na seriovy port
LOOP3	CALL	FREAD
	banksel	EEDATA
	MOVF	EEDATA,W
	banksel	KKOD
	MOVWF	KKOD
	banksel	EEDATH
	MOVF	EEDATH,W
	banksel	KKOD+1
	MOVWF	KKOD+1
	BTFSC	KKOD+1,5	; rozpoznani prazdneho zaznamu
	GOTO	KL_STOP

	MOVF	KKOD,F
	BTFSS	STATUS,Z
	GOTO	KL_SKOK1
	MOVF	KKOD+1,F
	BTFSS	STATUS,Z
	GOTO	KL_SKOK1
	GOTO	KL_STOP

KL_SKOK1	CALL	VYSLIKEY

	CALL	FREAD
	banksel	EEDATA
	MOVF	EEDATA,W
	banksel	KKOD
	MOVWF	KKOD
	banksel	EEDATH
	MOVF	EEDATH,W
	banksel	KKOD+1
	MOVWF	KKOD+1

	MOVF	KKOD,F
	BTFSS	STATUS,Z
	GOTO	KL_SKOK2
	MOVF	KKOD+1,F
	BTFSS	STATUS,Z
	GOTO	KL_SKOK2
	GOTO	KL_STOP

KL_SKOK2	CALL	CEKEJ
	GOTO	LOOP3
KL_STOP	BCF	PREPINAC	; sepnuti klavesnice
	BUFFINI 	BUFF_IN, BUFF_OUT,BUFF_STAT ; resetovani bufferu
	BSF	INTCON,GIE ; povoleni preruseni
	RETURN


;-----------------------
; |||||||| DATA ||||||||
;-----------------------

TABULKY	DATA	NAPIS1
	DATA	NAPIS2
	DATA	NAPIS3
	DATA	NAPIS4
	DATA	NAPIS5
	DATA	NAPIS6

NAPIS1	POS_D	D'0'
	DE	"vygenerujte novy"
	POS_D	D'64'
	DE	"!!  IR - kod  !!"
	EOFS
NAPIS2	POS_D	D'0'
	DE	" kam to ulozit? "
	POS_D	D'64'
	DE	" do zaznamu:"
	PRINTV	KINDEX,B'00111',0,0,0,1
	DBLOK	' ',D'3'
	EOFS
NAPIS3	DRESET
	DE	"otestujte prijem"
	EOFS
NAPIS4	POS_D	D'64'
	PRINTV	KINDEX,B'00111',0,0,0,1
	DBLOK	' ',D'3'
	EOFS
NAPIS5	POS_D	D'0'
	DE	"generujte klave-"
	POS_D	D'64'
	DE	" sovou posloup. "
	EOFS
NAPIS6	POS_D	D'0'
	DE	"ulozit na index:"
	POS_D	d'64'
	DE	"            "
	PRINTV	INDEX,B'00111',0,0,0,1
	DBLOK	' ',D'3'
	EOFS

; uzivatelskou FLASH pamet nejprve zaplnime jednickami - to indikuje prazdny zaznam!

FILL_	MACRO	BLOK_ADR_, POCET_
	ORG	BLOK_ADR_
	FILL	H'3FFF', POCET_
	ENDM

	FILL_	CK_ADR, D'650'
	FILL_	KEY_ADR, D'650'

	END
